@@ -122,6 +122,152 @@ sub schema_init($)
     $self->commit();
     return(1);
 }
+sub create_table_indexes($$)
+{
+    my ($self, $table)=@_;
+
+    if ( $table eq 'Titles' ) {
+	$self->runSQL("CREATE INDEX Titles_Idx1 on Titles (Year)");
+	$self->runSQL("CREATE INDEX Titles_Idx2 on Titles (SearchTitle)");
+	$self->runSQL("CREATE INDEX Titles_Idx3 on Titles (ParentID)");
+	return(1);
+    }
+    elsif ( $table eq 'Directors') {
+	$self->runSQL("CREATE INDEX Directors_Idx1 on Directors (SearchName)");
+	return(1);
+    }
+    elsif ( $table eq 'Actors') {
+	$self->runSQL("CREATE INDEX Actors_Idx1 on Actors (SearchName)");
+	$self->runSQL("CREATE INDEX Actors_Idx2 on Actors (Name)");
+	$self->runSQL("CREATE INDEX Actors_Idx3 on Titles2Actors (TitleID)");
+	$self->runSQL("CREATE INDEX Actors_Idx4 on Titles2Hosts (TitleID)");
+	$self->runSQL("CREATE INDEX Actors_Idx5 on Titles2Narrators (TitleID)");
+	return(1);
+    }
+    elsif ( $table eq 'Genres') {
+	$self->runSQL("CREATE INDEX Genres_Idx1 on Titles2Genres (TitleID)");
+	return(1);
+    }
+    elsif ( $table eq 'Ratings') {
+	$self->runSQL("CREATE INDEX Ratings_Idx1 on Ratings (TitleID)");
+	return(1);
+    }
+    elsif ( $table eq 'Keywords') {
+	$self->runSQL("CREATE INDEX Keywords_Idx1 on Titles2Keywords (TitleID)");
+	return(1);
+    }
+    elsif ( $table eq 'Plots' ) {
+	$self->runSQL("CREATE INDEX Plots_Idx1 on Plots (TitleID)");
+	return(1);
+    }
+    return(0);
+}
+
+sub drop_table_indexes($$)
+{
+    my ($self, $table)=@_;
+
+    if ( $table eq 'Titles' ) {
+	$self->runSQL("DROP INDEX IF EXISTS Titles_Idx1");
+	$self->runSQL("DROP INDEX IF EXISTS Titles_Idx2");
+	$self->runSQL("DROP INDEX IF EXISTS Titles_Idx3");
+	return(1);
+    }
+    elsif ( $table eq 'Directors') {
+	$self->runSQL("DROP INDEX IF EXISTS Directors_Idx1");
+	return(1);
+    }
+    elsif ( $table eq 'Actors') {
+	$self->runSQL("DROP INDEX IF EXISTS Actors_Idx1");
+	$self->runSQL("DROP INDEX IF EXISTS Actors_Idx2");
+	$self->runSQL("DROP INDEX IF EXISTS Actors_Idx3");
+	$self->runSQL("DROP INDEX IF EXISTS Actors_Idx4");
+	$self->runSQL("DROP INDEX IF EXISTS Actors_Idx5");
+	return(1);
+    }
+    elsif ( $table eq 'Genres') {
+	$self->runSQL("DROP INDEX IF EXISTS Genres_Idx1");
+	return(1);
+    }
+    elsif ( $table eq 'Ratings') {
+	$self->runSQL("DROP INDEX IF EXISTS Ratings_Idx1");
+	return(1);
+    }
+    elsif ( $table eq 'Keywords') {
+	$self->runSQL("DROP INDEX IF EXISTS Keywords_Idx1");
+	return(1);
+    }
+    elsif ( $table eq 'Plots' ) {
+	$self->runSQL("DROP INDEX IF EXISTS Plots_Idx1");
+	return(1);
+    }
+    return(0);
+}
+
+# Convert a title into a searchtitle by lowercasing,
+# making it ASCII and removing punctuation.
+#
+sub makeSearchableTitle($;$;$) {
+  my ($self, $str) = @_;
+  return lc $self->RemovePunctuation(lc( CharsetMap( $str ) ));
+}
+
+
+use Text::Unidecode;
+
+#   All characters outside the ASCII range (0x00-0x7F) are replaced by ASCII equivalents,
+#    using function Text::Unidecode::unidecode
+#
+sub CharsetMap($) {
+  my ($str) = @_;
+  
+  # do replacements that unidecode doesn't know about (or does wrong)
+  ### IT WOULD BE NICE IF THESE WERE IN A TABLE
+  $str =~ s/\x{0133}/ij/og;     # 'ij' -> ij             ("" in unidecode)
+  $str =~ s/\x{20ac}/EUR/og;    # euro symbol -> EUR     (EU in unidecode)
+  $str =~ s/\x{2122}/TM/og;     # trademark symbol -> TM ("" in unidecode)
+  $str =~ s/\x{a3}/GBP/og;      # pound sign -> GBP (PS in unidecode)
+  
+  # now do the real decode
+  $str = unidecode($str);
+        
+  #print "[$str]\n" if ($debug);
+ 
+  return ($str);
+}
+
+
+my @punctuation;
+
+# Function that removes all punctuation and whitespace from a string.
+#  '&' is converted to 'and' along the way
+sub RemovePunctuation($;$;$)
+{
+    my ($self, $str) = @_;
+
+    # Load the array of hashes that contain the punctuation
+    # replacements in priority order
+    if ( !@punctuation ) {
+        my @plist = @{$self->select2Matrix("select priority,pattern,replacement from Punctuation order by priority")};
+        my $cnt = 0;
+        foreach my $p (@plist) {
+            my $pattern = $p->[1];
+            my $compiled = qr/$pattern/i;
+            $punctuation[$cnt]{origpattern} = $pattern;
+            $punctuation[$cnt]{pattern}     = $compiled;
+            $punctuation[$cnt]{replacement} = $p->[2];
+            $cnt++;
+        }
+    }
+
+    foreach my $ref (@punctuation) {
+        #print "[$str] $ref->{origpattern} " if ($debug);
+        $str =~ s/$ref->{pattern}/$ref->{replacement}/g;
+        #print "[$str]\n" if ($debug);
+    }
+    #print "<$str>\n--------\n" if ($debug);
+    return ($str);
+}
 
 =head1 INHERITED FUNCTIONS
 
@@ -276,8 +422,6 @@ the web interface at L<http://rt.cpan.org/NoAuth/ReportBug.html?Queue=IMDB-Local
 automatically be notified of progress on your bug as I make changes.
 
 
-
-
 =head1 SUPPORT
 
 You can find documentation for this module with the perldoc command.
@@ -512,6 +656,9 @@ CREATE TABLE Titles2Directors (
     DirectorID INTEGER
 );
 
+--
+-- Actors are used for Roles, Hosts and Narrators
+-- 
 CREATE TABLE Actors (
     ActorID INTEGER PRIMARY KEY AUTOINCREMENT,
     SearchName VARCHAR,
@@ -521,9 +668,17 @@ CREATE TABLE Actors (
 CREATE TABLE Titles2Actors (
     TitleID INTEGER,
     ActorID INTEGER,
-    Billing INTEGER,
-    IsHost INTEGER,
-    IsNarrator INTEGER
+    Billing INTEGER
+);
+
+CREATE TABLE Titles2Hosts (
+    TitleID INTEGER,
+    ActorID INTEGER
+);
+
+CREATE TABLE Titles2Narrators (
+    TitleID INTEGER,
+    ActorID INTEGER
 );
 
 CREATE TABLE Genres (
@@ -554,7 +709,10 @@ CREATE TABLE Titles2Keywords (
 );
 
 CREATE TABLE Plots (
+    PlotID INTEGER PRIMARY KEY AUTOINCREMENT,
     TitleID INTEGER,
-    Description VARCHAR
+    Sequence INTEGER,
+    Description VARCHAR,
+    Author VARCHAR
 );
 
